require 'teamforge'

##
#  The ApplicationController in a Rails application inherits
#  from ActionController::Base.
#  All other controllers in turn inherit from ApplicationController.
#  This gives you one class to configure things such as
#  request forgery protection and filtering of sensitive request parameters.
#
# rubocop:disable Metrics/ClassLength
# disabling the metrics as this class will be big by design
class ApplicationController < ActionController::Base
  # Harish : read more on the following
  protect_from_forgery with: :null_session
  rescue_from Exceptions::PermissionDenied, with: :render_permission_denied
  rescue_from Exceptions::Unauthorized, with: :render_unauthorized
  rescue_from Exceptions::NoSuchObjectId, with: :render_no_such_object_id

  CTF_BASE_URL = Rails.configuration.teamforge.fetch :ctf_base_url
  TeamForge.use_ssl = CTF_BASE_URL.start_with?('https:')
  TeamForge.server = CTF_BASE_URL

  ##
  #  Authenticates the request against teamforge .
  #  Use the existing session if the session is valid unless we have a new
  #  one time token.
  #  In cases of
  #  1. invalid session
  #  2. unexpected error in session validation
  #     (may be due to failure of some other case)
  #  3. session itself does not exist for that request yet
  #
  #  try the token based authentication.
  #  If token based authentication also fails, redirect him
  #  to teamforge '/sf/sfmain/do/login' page.

  protected

  attr_reader :ctf_project_path, :ctf_project_url_name

  def authenticate
    Rails.logger.debug 'Authenticating the request...'

    begin
      if params[:sfLoginToken] &&
          session['ctf_one_time_token'] != params[:sfLoginToken]
        session['ctf_one_time_token'] = params[:sfLoginToken]
        authenticate_with_token
        Rails.logger.debug 'Authenticated with one_time_token'
        # we give an exception to go url as this is the only url
        # without project context
        unless @_request.path_info.start_with?(
            "/#{Rails.configuration.teamforge['app_url_prefix']}/go")
          redirect_to query_string_flushed_url
        end
      elsif session['ctf_soapsession_id']
        authenticate_with_session_id
        Rails.logger.debug 'Authenticated with session id'
      else
        fail Exceptions::Unauthorized
      end
    rescue TeamForge::TeamForgeError => e
      Rails.logger.debug 'SOAP Call Failed so redirecting to teamforge'
      Rails.logger.debug e.message
      raise Exceptions::Unauthorized
    rescue => e
      Rails.logger.debug 'Authentication Failed so redirecting to teamforge'
      Rails.logger.debug e.message
      raise Exceptions::Unauthorized
    end
    Rails.logger.debug @ctf_soapsession_id if Rails.env == 'development'
  end

  def authenticate_with_token
    @ctf_soapsession_id = TeamForge::GetSessionId
    .new(params[:sfLoginToken])
    .request
    @ctf_project_path = project_path
    @ctf_project_url_name = @ctf_project_path.gsub('projects.', '').to_s
    @ctf_prpl_id = TeamForge::PluggableApp::GetLinkPlugId
    .new(@ctf_soapsession_id,
         @ctf_project_path,
         Rails.configuration.teamforge[:iaf_base_url]
    )
    .request
    user_details = TeamForge::GetCurrentUserData
    .new(@ctf_soapsession_id).request
    @username = user_details.username
    @is_super_user = user_details.super_user == 'true'
    session['ctf_soapsession_id'] = @ctf_soapsession_id
    @ctf_iframe_url = get_iframe_url @ctf_project_path, @ctf_prpl_id
  end

  def authenticate_with_session_id
    @ctf_soapsession_id = session[:ctf_soapsession_id]
    @ctf_project_path = project_path
    @ctf_project_url_name = @ctf_project_path.gsub('projects.', '').to_s
    @ctf_prpl_id = TeamForge::PluggableApp::GetLinkPlugId
    .new(session[:ctf_soapsession_id],
         @ctf_project_path,
         Rails.configuration.teamforge[:iaf_base_url]
    )
    .request
    @ctf_iframe_url = get_iframe_url @ctf_project_path, @ctf_prpl_id
    user_details = TeamForge::GetCurrentUserData
    .new(@ctf_soapsession_id).request
    @username = user_details.username
    @is_super_user = user_details.super_user == 'true'
  end

  def authenticate_iaf_api
    TeamForge::GetSessionId.new(params[:token]).request
  end

  def authenticate_with_soap_token
    begin
      @ctf_soapsession_id = request.headers['X-Binaries-Session']
      @ctf_project_path = params['tf_project_path']
      user_details = TeamForge::GetCurrentUserData
      .new(@ctf_soapsession_id).request
      @username = user_details.username
      @is_super_user = user_details.super_user == 'true'
    rescue TeamForge::TeamForgeError => e
      Rails.logger.debug 'SOAP Call Failed so redirecting to teamforge'
      Rails.logger.debug e.message
      raise Exceptions::Unauthorized
    rescue => e
      Rails.logger.debug 'Authentication Failed so redirecting to teamforge'
      Rails.logger.debug e.message
      raise Exceptions::Unauthorized
    end
  end


  def set_repo_privileges
    @project_id = TeamForge::GetProjectByPath
    .new(@ctf_soapsession_id, @ctf_project_path).request['id']
    permissions = Services::AuthorizationService
    .get_permissions @ctf_soapsession_id, @project_id
    args = [@ctf_soapsession_id, @project_id, permissions]
    @can_view_repo = Services::AuthorizationService
    .read_repo_permission_in_project?(*args)
    @can_create_repo = Services::AuthorizationService
    .create_repo_permission_in_project?(*args)
    @can_edit_repo = Services::AuthorizationService
    .edit_repo_permission_in_project?(*args)
    @can_delete_repo = Services::AuthorizationService
    .delete_repo_permission_in_project?(*args)
  end

  def project_path
    if params[:sfProj]
      return 'projects.' + params[:sfProj]
    elsif params[:sfId]
      return TeamForge::PluggableApp::GetProjectPathByIntegratedAppId
      .new(@ctf_soapsession_id,
           params[:sfId])
      .request
    elsif params[:object_id]
      repository_record = Repository.where(id: params[:object_id].to_i,
                                           is_deleted: false)
      if repository_record.empty?
        fail Exceptions::NoSuchObjectId
      else
        return repository_record.first.project_url_name
      end
    end
  end

  def get_iframe_url(ctf_project_path, ctf_prpl_id)
    CTF_BASE_URL +
        '/sf/sfmain/do/topInclude/' +
        ctf_project_path +
        '?linkId=' +
        ctf_prpl_id
  end

  def teamforge_login_url
    CTF_BASE_URL + '/sf/sfmain/do/login'
  end

  def teamforge_object_id_not_found_url
    # want to redirect to teamforge object not found page
    # but as such a separate page does not exists
    # I am attaching a /notfound to the end to simulate a
    # unexisting object id
    CTF_BASE_URL + '/sf/go/' +
        Services::SoapService.get_binary_app_prefix(@ctf_soapsession_id) +
        '_' +
        params[:object_id] + '/notfound'
  end

  ##
  #  sets the locale to the best fit among the available locales.
  #  This methodology aligns to how teamforge does internationalization.
  #  Teamforge supports english, chinese, japanese, korean.
  #  Note: For chinese as teamforge does not have human translators
  #  they have it in english itself.
  #  But our app gives provision replacing text later with chinese.

  def set_locale
    teamforge_supported_languages = %w(en ja ko zh)
    I18n.locale = http_accept_language
    .compatible_language_from(teamforge_supported_languages)
  end

  def render_permission_denied
    render file: 'public/403.html', status: 403
  end

  def render_unauthorized
    redirect_to teamforge_login_url
  end

  def render_no_such_object_id
    # render file: 'public/no_such_object_id.html'
    redirect_to teamforge_object_id_not_found_url
  end

  helper_method :one_time_token
  def one_time_token
    TeamForge::GetOneTimeToken.new(@ctf_soapsession_id).request
  end

  helper_method :url_root
  def url_root
    @url_root ||= Rails.configuration.relative_url_root.chomp('/')
  end

  helper_method :app_root
  def app_root
    @app_root ||=
        url_root + '/' + Rails.configuration.teamforge['app_url_prefix']
  end

  helper_method :full_app_root
  def full_app_root
    Rails.configuration.teamforge[:ctf_base_url] + '/' + app_root
  end

  helper_method :list_repo_page_url
  def list_repo_page_url
    @list_repo_page_url = "#{app_root}/projects.#{@ctf_project_url_name}"
  end

  helper_method :query_string_flushed_url
  def query_string_flushed_url
    @query_string_flushed_url = url_root +
        @_request.path_info.chomp('/') +
        "/projects.#{@ctf_project_url_name}"
  end
end
# rubocop:enable Metrics/ClassLength
