require 'ctf/orchestrate_client'
##
# configures report for repositories
# in a project.
#
class ReportController < ApplicationController
  before_action :authenticate, :set_locale, :check_permission

  def new
    @report = Report.new
    set_object_id
    get_repository @repository_id
    fetch_build_sources

    @action = app_root + '/report'
    @method = 'POST'
    render partial: 'form', action: 'create'
  end

  def create
    @report = Report.new
    @repository_id = params['report']['repository_id']

    respond_to do |format|
      begin
        get_repository @repository_id
        @project = @repository.project

        # Post schema to orchestrate
        schema =
            Services::OrchestrateService.schema_exists?(@ctf_soapsession_id) ||
                Services::OrchestrateService.create_schema(@ctf_soapsession_id) or
            fail 'Unable to check / create schema'

        unless @project.tool_ref
          # Fetch or create tool
          tool = Services::OrchestrateService.tool_exists?(
              @ctf_soapsession_id, @ctf_project_url_name) ||
              Services::OrchestrateService.create_tool(
                  @ctf_soapsession_id, @ctf_project_url_name)
          @project.tool_ref = tool['tool']['id']
          @project.save!
        end

        # Create source
        associated_source = params[:associated_source]
        associated_source = nil if associated_source == 'none'
        source = Services::OrchestrateService.create_source(
            @ctf_soapsession_id,
            @ctf_project_url_name, @repository.repository_name,
            @project.tool_ref, schema[0]['custom_schema_store_id'],
            associated_source)
        source = source['source']
        @report.repository_id = @repository_id
        @report.source_ref = source['id']
        @report.source_association_key = source['source_association_key']
        @report.queue_server = source['queue_server']
        @report.queue_username = source['queue_username']
        @report.queue_password = source['queue_password']
        @report.queue_name = 'orchestrate.custom'

        # Post to nexus
        params[:sessionId] = @ctf_soapsession_id
        params[:iafappname] = @repository.iaf_app_name
        params[:projectId] = @project_id
        params[:repoId] = @repository.repository_id
        params[:object_id] = "binary_artifact_#{@report.repository_id}"
        params[:source_association_key] = @report.source_association_key
        params[:queue_server] = @report.queue_server
        params[:queue_username] = @report.queue_username
        params[:queue_password] = @report.queue_password
        params[:queue_name] = @report.queue_name
        Services::RepositoryService.report_orchestrate_config params

        # Save to database
        if @report.save
          format.json do
            render json: 'Successfully configured traceability for repository',
                   status: :created,
                   location: @report
            @action = "#{app_root}/report/#{@repository_id}"
            @method = 'put'
          end

        else
          fail 'Error saving row'
        end
      rescue => e
        Rails.logger.error "Error creating source: #{e}"
        format.json do
          render json: 'Error: Unable to create source',
                 status: 400
        end
      end
    end
  end

  def edit
    @report = Report.find_by_repository_id(params[:id])
    set_object_id
    # if record does not exists it is an new call
    # else a edit call (for reset)
    if @report.nil?
      redirect_to full_app_root + '/report/new?sfProj=' +
                      params[:sfProj] +
                      '&id=' + params[:id],
                  status: 301
      return
    else
      begin
        fetch_build_sources
        get_repository @repository_id
        refresh_source
      rescue => e
        Rails.logger.warn "Exception occurred: #{e}"
      ensure
        @action = "#{app_root}/report/#{@repository_id}"
        @method = 'put'
        render partial: 'form'
      end
    end
  end

  def destroy
    set_object_id
    get_repository @repository_id
    respond_to do |format|
      begin
        @report = Report.find_by_repository_id(params[:id])
        Services::OrchestrateService.deactivate_source(@ctf_soapsession_id,
                                                       @ctf_project_url_name,
                                                       @report.source_ref)
        msg = 'Successfully deleted configuration for repository'
        response_code = 200
      rescue => e
        Rails.logger.error "Error deleting source: #{e}"
        msg = 'Warning: deactivation of source failed.
              Successfully deleted the configuration for repository'
        response_code = 200
      end
      begin
        params[:sessionId] = @ctf_soapsession_id
        params[:iafappname] = @repository.iaf_app_name
        params[:projectId] = @project_id
        params[:repoId] = @repository.repository_id
        if @repository.repository_provider.start_with? 'Nexus'
          Services::RepositoryService.delete_report_orchestrate_config params
        end
        Report.where(repository_id: params[:id]).destroy_all
      rescue => ex
        Rails.logger.error "Error deleting source: #{ex}"
        msg = 'Error: Unable to delete configuration'
        response_code = 400
      end
      format.json do
        render json: msg,
               status: response_code
      end
    end
  end

  def update # activate / deactivate / change associated source
    @report = Report.find_by_repository_id(params[:id])
    set_object_id
    get_repository @repository_id

    action = ''
    respond_to do |format|
      begin
        case params[:update_type]
          when 'deactivate_source'
            Services::OrchestrateService.deactivate_source(
                @ctf_soapsession_id, @ctf_project_url_name, @report.source_ref)
            @report.is_active = false
            action = 'de-activated'
          when 'reactivate_source'
            Services::OrchestrateService.reactivate_source(
                @ctf_soapsession_id, @ctf_project_url_name, @report.source_ref)
            @report.is_active = true
            action = 're-activated'
          when 'update_associated_source'
            associated_source = params[:associated_source]
            associated_source = nil if associated_source == 'none'
            Services::OrchestrateService.update_associated_source(
                @ctf_soapsession_id, @ctf_project_url_name, @report.source_ref,
                associated_source)
            @report.associated_source_ref = associated_source
            action = 'updated'
        end

        @report.save!
        format.json do
          render json: "Successfully #{action} traceability for repository",
                 status: 200
        end
      rescue => e
        rails.logger.error "error updating source: #{e}"
        format.json do
          render json: 'Error: Unable to save configuration',
                 status: 400
        end
      end
    end
  end

  def set_object_id
    @repository_id = params[:id]
  end

  private

  def check_permission
    @project_id = TeamForge::GetProjectByPath
    .new(@ctf_soapsession_id, @ctf_project_path).request['id']
    can_create_repo = Services::AuthorizationService
    .create_repo_permission_in_project? @ctf_soapsession_id, @project_id

    fail Exceptions::PermissionDenied unless can_create_repo
  end

  def get_repository(id)
    @repository = Repository.find_by_id id
  end

  def fetch_build_sources
    @build_sources = Services::OrchestrateService.fetch_build_sources(
        @ctf_soapsession_id, @ctf_project_url_name) || []
    @build_sources.unshift('id' => 'none', 'display_name' => 'None')
  end

  def refresh_source
    orc_source =
        if @report.source_ref.nil?
          Services::OrchestrateService.fetch_source_by_sak(
              @ctf_soapsession_id,
              @ctf_project_url_name,
              @report.source_association_key)
        else
          Services::OrchestrateService.fetch_source(@ctf_soapsession_id,
                                                    @ctf_project_url_name,
                                                    @report.source_ref)
        end
    orc_source and @report.tap do |s|
      s.associated_source_ref = orc_source['associated_activity_source_id']
      s.is_active = orc_source['active']
      s.queue_username = orc_source['queue_username']
      s.queue_password = orc_source['queue_password']
      s.queue_server = orc_source['queue_server']
      s.source_association_key = orc_source['source_association_key']
      s.associated_source_ref = orc_source['associated_activity_source_id']

      s.save! if s.changed?
    end
  end
end
