require 'rest_client'
require 'json'
require 'teamforge'
require 'uri'
require 'services/repository_service'
##
#  Controller to execute the actions on repositories like
#  1.create repository,
#  2.list repositories,
#  3.delete repository
class RepositoryController < ApplicationController
  before_action :authenticate, :set_locale, :set_repo_privileges , :list_binary_app

  attr_accessor :app_statuses

  ##
  # list repositories controller action
  #
  def show
    @all_repositories  = []
    @count_statistic = 0
    fail Exceptions::PermissionDenied unless @can_view_repo
    fail Exceptions::NoPluggableBinaryApps if @list_of_binary_apps.count == 0
    sync
    @base_app_prefix = Services::SoapService
    .get_binary_app_prefix @ctf_soapsession_id
    @list_of_binary_apps.each do |binary_app|
      repository_list_from_db = Repository
      .where(project_url_name: @ctf_project_path.to_s,
             iaf_app_name: binary_app['name'].to_s,
             is_deleted: false)
      status = @app_statuses[binary_app['name'].to_s] || 'Down'
      repository_list_from_db.each do |repository_from_db|
        repository = {}
        repository['object_id'] = repository_from_db['id'].to_s
        repository['server_status'] = status
        repository['name'] = repository_from_db['repository_name']
        repository['id'] = repository_from_db['repository_id']
        repository['format'] = repository_from_db['repository_format']
        repository['repoType'] = repository_from_db['repository_type']
        repository['repoPolicy'] = repository_from_db['repository_policy']
        repository['repoHost'] = repository_from_db['repository_host']
        repository['iaf_app_name'] = repository_from_db['iaf_app_name']
        @all_repositories.push(repository)
      end
    end
    @all_repositories.sort_by! { |repo| [repo['name'], repo['iaf_app_name']] }
  rescue Exceptions::PermissionDenied
    flash[:alert] = 'You do not have the necessary permissions. ' +
        'Please contact your project administrator'
  rescue Exceptions::NoPluggableBinaryApps
    render 'repository/landing_page'
  end

  ##
  # list orphaned repositories controller action
  #
  def show_linkable
    @linkable_repositories  = []
    @count_statistic = 0
    fail Exceptions::PermissionDenied unless @is_super_user
    fail Exceptions::NoPluggableBinaryApps if @list_of_binary_apps.count == 0
    i = 0
    repository_list_from_providers = remote_repo_data :list_repositories
    repository_list_from_providers.each do |app_name, repo_list|
      repo_list.data.sort_by { |r| r['name'] } .each do |repo_from_provider|
        next if Repository.where(
            iaf_app_name: app_name,
            repository_name: repo_from_provider['name'],
            is_deleted: false).present?
        repository = {}
        repository['seq'] = format '%05d', (i += 1)
        repository['name'] = repo_from_provider['name']
        repository['id'] = repo_from_provider['id']
        repository['format'] = repo_from_provider['format']
        repository['repoType'] = repo_from_provider['repoType']
        repository['repoPolicy'] = repo_from_provider['repoPolicy']
        repository['repoHost'] = repo_from_provider['contentResourceURI']
        repository['iafappname'] = app_name
        @linkable_repositories.push(repository)
      end
    end
  rescue Exceptions::PermissionDenied, Exceptions::NoPluggableBinaryApps
    render 'repository/landing_page'
  end

  ##
  # create repository controller action
  #
  def create
    if request.post?
      create_post
    else
      create_get
    end
  end

  ##
  # delete repository controller action
  #
  def destroy
    params['sessionId'] = @ctf_soapsession_id
    repo = Repository.find(params['_listItem'].to_i)
    params[:projectId] = @project_id
    params[:repoId] = repo.repository_id
    param_hash = { status: 301 }
    if  repo.repository_provider.start_with? 'Nexus'
      response = Services::RepositoryService.delete_repository params
      if response.success
        repo.is_deleted = true
        repo.save!
        param_hash[:notice] = t('repository_deleted_notice')
      else
        param_hash[:alert] = t('repository_delete_failed_alert')
      end
    else
      repo.is_deleted = true
      repo.save!
      param_hash[:notice] = t('repository_deleted_notice')
    end
    redirect_to list_repo_page_url,param_hash
  end

  ##
  # link repository controller action
  #
  def link
    create_project_if_required
    suffix = params['_listItem']
    params['sessionId'] = @ctf_soapsession_id
    params[:projectId] = @project_id
    params[:repoId] = params["repoId_#{suffix}"]
    params[:iafappname] = params["iafappname_#{suffix}"]
    params['prodType'] = get_iaf_details(params["iafappname"])['type']
    params[:username] = @username
    resp = Services::RepositoryService.link_repository params
    klass = "#{params['prodType'].capitalize}Repository".constantize
    param_hash = { status: 301 }
    if resp.success
      repo = klass.new do |r|
        r.project_url_name = @ctf_project_path.to_s
        r.iaf_app_name = params["iafappname_#{suffix}"]
        r.repository_id = resp.data['id'] || params["repoId_#{suffix}"]
        r.repository_name = resp.data['name'] ||params["repoName_#{suffix}"]
        r.repository_format = resp.data['format'] || params["repoFormat_#{suffix}"]
        r.repository_type = resp.data['repoType'] || params["repoType_#{suffix}"]
        r.repository_policy = resp.data['repoPolicy'] || params["repoPolicy_#{suffix}"]
        r.repository_host = resp.data['contentResourceURI'] ||params["repoHost_#{suffix}"]
        r.is_deleted = false
      end
      repo.save!
      param_hash[:notice] = t('repository_linked_notice')
      redirect_to list_repo_page_url, param_hash
    else
      param_hash[:alert] = t('repository_link_failed_alert')
      redirect_to "#{app_root}/listLinkableRepositories/" +
                      'projects.' + params[:sfProj],param_hash
    end
  end

  # Helper functions
  def create_post
    create_project_if_required
    params['sessionId'] = @ctf_soapsession_id
    params['username'] = @username
    params['prodType'] = get_iaf_details(params['iafappname'])['type']
    resp = Services::RepositoryService.create_repository params
    repo_class = "#{params[:prodType].capitalize}Repository".constantize
    if resp.success?
      repo_new_entry = repo_class.new(
          project_url_name: @ctf_project_path.to_s,
          iaf_app_name: params['iafappname'].to_s,
          repository_id: resp.data['id'].to_s,
          repository_name: resp.data['name'].to_s,
          repository_format: resp.data['format'].to_s,
          repository_type: resp.data['repoType'].to_s,
          repository_policy: resp.data['repoPolicy'].to_s,
          repository_host: resp.data['contentResourceURI'],
          is_deleted: false
      )
      repo_new_entry.save
      redirect_to list_repo_page_url,
                  notice: resp.message,
                  status: 301

    else
      redirect_to "#{app_root}/createRepository/" +
                      'projects.' + params[:sfProj] +
                      '?&selectedserver=' + params['iafappname'] +
                      '&selectedrepotype=' + params['repoType'],
                  notice: resp.message,
                  status: 301
    end
  rescue StandardError => e
    Rails.logger.error "Error creating repository: #{e}"
    redirect_to "#{app_root}/createRepository/" +
                    'projects.' + params[:sfProj] +
                    '?&selectedserver=' + params['iafappname'] +
                    '&selectedrepotype=' + params['repoType'],
                notice: 'Error occurred creating repository',
                status: 301
  end

  def create_get
    fail Exceptions::PermissionDenied unless @can_create_repo
    iaf_name = params['selectedserver'] ? params['selectedserver']:@list_of_binary_apps[0]['name']
    prodType = get_iaf_details(iaf_name)['type']
    case prodType
      when 'nexus'
        render_nexus_create_repository_ui
      when 'artifactory'
        render_artifactory_create_repository_ui
      when 'archiva'
        render_archiva_create_repository_ui
      else
        render 'repository/others/create'
    end
  end

  def render_nexus_create_repository_ui
    if params['selectedserver']
      rearrange_pluggable_apps
      nexus_maven_repos params['selectedserver']
    else
      nexus_maven_repos @list_of_binary_apps[0]['name']
    end
    case params['selectedrepotype']
      when 'hosted'
        @repo_types = [{ id: 'hosted', value: 'Hosted Repository' },
                       { id: 'proxy', value: 'Proxy Repository' },
                       { id: 'virtual', value: 'Virtual Repository' }]
      when 'proxy'
        @repo_types = [{ id: 'proxy', value: 'Proxy Repository' },
                       { id: 'hosted', value: 'Hosted Repository' },
                       { id: 'virtual', value: 'Virtual Repository' }]
      when 'virtual'
        @repo_types = [{ id: 'virtual', value: 'Virtual Repository' },
                       { id: 'hosted', value: 'Hosted Repository' },
                       { id: 'proxy', value: 'Proxy Repository' }]
      else
        @repo_types = [{ id: 'hosted', value: 'Hosted Repository' },
                       { id: 'proxy', value: 'Proxy Repository' },
                       { id: 'virtual', value: 'Virtual Repository' }]
    end
    render 'repository/nexus/create'
  end

  def render_artifactory_create_repository_ui
    if params['selectedserver']
      rearrange_pluggable_apps
      artifactory_repos params['selectedserver']
    else
      artifactory_repos @list_of_binary_apps[0]['name']
    end
    case params['selectedrepotype']
      when 'local'
        @repo_types = [{ id: 'local', value: 'Local Repository' },
                       { id: 'remote', value: 'Remote Repository' },
                       { id: 'virtual', value: 'Virtual Repository' }]
      when 'remote'
        @repo_types = [{ id: 'remote', value: 'Remote Repository' },
                       { id: 'local', value: 'Local Repository' },
                       { id: 'virtual', value: 'Virtual Repository' }]
      when 'virtual'
        @repo_types = [{ id: 'virtual', value: 'Virtual Repository' },
                       { id: 'local', value: 'Local Repository' },
                       { id: 'remote', value: 'Remote Repository' }]
      else
        @repo_types = [{ id: 'local', value: 'Local Repository' },
                       { id: 'remote', value: 'Remote Repository' },
                       { id: 'virtual', value: 'Virtual Repository' }]
    end
    render 'repository/artifactory/create'
  end

  def render_archiva_create_repository_ui
    render 'repository/archiva/create'
  end

  def nexus_maven_repos(selected_server)
    @maven1repos = Repository
    .where(project_url_name: @ctf_project_path.to_s,
           iaf_app_name: selected_server,
           repository_format: 'maven1',
           is_deleted: false)
    @maven2repos = Repository
    .where(project_url_name: @ctf_project_path.to_s,
           iaf_app_name: selected_server,
           repository_format: 'maven2',
           is_deleted: false)
  end

  def artifactory_repos(selected_server)
    @all_artifactory_repo = Repository
    .where(project_url_name: @ctf_project_path.to_s,
           iaf_app_name: selected_server,
           is_deleted: false)
  end

  def rearrange_pluggable_apps
    @list_of_binary_apps.each_with_index do |pluggable_app, index|
      next unless pluggable_app['name'] == params['selectedserver']
      @list_of_binary_apps .insert(0, @list_of_binary_apps.delete_at(index))
      return
    end
  end

  protected

  def sync
    remote_list_hash = remote_repo_data :list_repositories_for_project
    remote_list_hash.each do |app_name, repo_list|
      Repository.where(project_url_name: @ctf_project_path.to_s,
                       iaf_app_name: app_name.to_s,
                       is_deleted: false).each do |db_repo|
        remote_repo = repo_list.data.find do |repo|
          repo['id'] == db_repo.repository_id
        end
        if remote_repo.nil? # deleted
          db_repo.is_deleted = true
        end
        db_repo.save! if db_repo.changed?
      end
    end
  end

  def list_binary_app
    @list_of_binary_apps= JSON.parse(
        Services::SoapService.list_of_binaries @ctf_soapsession_id).each do|iaf_app|
      #Determining type i.e artifactory or nexus is currently based on the prefix's starting character
      #Currently there is no other mechanism available to get the type of a binary app.
      if iaf_app['prefix'].downcase.start_with? 'art'
        iaf_app['type'] = 'artifactory'
      else
        iaf_app['type'] = 'nexus'
      end
    end rescue []
  end

  private

  def remote_repo_data(method)
    params['sessionId'] = @ctf_soapsession_id
    params['username']= @username
    params[:projectId] = @project_id
    remote_list_hash = {}
    @list_of_binary_apps.each do |binary_app|
      app_name = binary_app['name'].to_s
      (@app_statuses ||= {})[app_name] = 'Down'
      params[:iafappname] = app_name
      params['prodType']= binary_app['type']
      (threads ||= []) << Thread.new do
        Rails.logger.info "Fetching repo list from app #{app_name}"
        remote_list = Services::RepositoryService.send method, params
        if remote_list.success && remote_list.data
          Rails.logger.info "Got repo list from app #{app_name}"
          @app_statuses[app_name] = 'Running'
          remote_list_hash[app_name] = remote_list
        end
      end
      threads.each(&:join)
    end
    remote_list_hash
  end

  def repo_exists(iaf_app)
    Repository.where(project_url_name: @ctf_project_path.to_s,iaf_app_name:iaf_app ).present?
  end

  def create_project_if_required
    Project.find_by_project_url_name(@ctf_project_path) ||
        Project.new do |proj|
          proj.project_url_name = @ctf_project_path
          proj.save!
        end
  end

  def get_iaf_details (iaf_name)
    @list_of_binary_apps.find{|iaf_app|
      iaf_app['name'] == iaf_name
    }
  end

end