##
# class to do operations related to nexus
#
class Services::Integrations::Nexus
  def self.construct_payload(params)
    # TODO: check validity of the project id in the api we introduced in nexus
    if params[:repoType] == 'hosted'
      return create_hosted_repo_payload params
    elsif params[:repoType] == 'virtual'
      return create_virtual_repo_payload params
    else
      return create_proxy_repo_payload params
    end
  end

  def self.create_repository(params)
    begin
      endpoint_url = Services::SoapService
        .get_endpoint_url_for_iaf_app params[:sessionId],
                                      params[:iafappname]
      payload = construct_payload params
      repositories_string = RestClient::Resource.new(
          format('%s/ctf/repositories-in-project/%s',
                 endpoint_url,
                 params[:projectId]),
          headers: http_headers(params[:sessionId]),
          verify_ssl: OpenSSL::SSL::VERIFY_NONE
      ).post payload.to_json
      repositories_json = JSON.parse(repositories_string)
      response = Status.new(true,
                            200,
                            message: 'Successfully created ' +
                              'Repository with Id ' +
                              repositories_json['data']['id'],
                            data: repositories_json['data'])
    rescue RestClient::ResourceNotFound, RestClient::RequestFailed => e
      response = rescue_from_request_failed(e)
    rescue Errno::ECONNREFUSED => e
      response = rescue_from_connection_error(e)
    end
    response
  end

  def self.create_hosted_repo_payload(params)
    if params[:provider] == 'site'
      provider_role = 'org.sonatype.nexus.proxy.repository.WebSiteRepository'
    else
      provider_role = 'org.sonatype.nexus.proxy.repository.Repository'
    end
    payload = {
      data: {
        id: params[:id],
        name: params[:name],
        provider: params[:provider],
        providerRole: provider_role,
        format: params[:format],
        repoType: params[:repoType],
        exposed: params[:exposed] == 'true',
        writePolicy: params[:writePolicy],
        browseable: params[:browseable] == 'true',
        indexable: params[:indexable] == 'true',
        notFoundCacheTTL: params[:notFoundCacheTTL].to_i,
        repoPolicy: params[:repoPolicy],
        defaultLocalStorageUrl: params[:defaultLocalStorageUrl]
      }
    }
    payload
  end

  def self.create_virtual_repo_payload(params)
    provider_role = 'org.sonatype.nexus.proxy.repository.ShadowRepository'
    payload = {
      data: {
        exposed: true, # this always was true for virtual repo
        id: params[:id],
        name: params[:name],
        provider: params[:provider],
        providerRole: provider_role,
        repoType: params[:repoType],
        shadowOf: params[:shadowOf],
        syncAtStartup: params[:syncAtStartup] == 'true'
      }
    }
    payload
  end

  def self.create_proxy_repo_payload(params)
    provider_role = 'org.sonatype.nexus.proxy.repository.Repository'
    payload = {
      data: {
        id: params[:id],
        name: params[:name],
        provider: params[:provider],
        providerRole: provider_role,
        format: params[:format],
        repoType: params[:repoType],
        exposed: params[:exposed] == 'true',
        writePolicy: 'READ_ONLY',
        browseable: params[:browseable] == 'true',
        indexable: params[:indexable] == 'true',
        notFoundCacheTTL: params[:notFoundCacheTTL].to_i,
        repoPolicy: params[:repoPolicy],
        defaultLocalStorageUrl: params[:defaultLocalStorageUrl],
        remoteStorage: {
          remoteStorageUrl: params[:remoteStorageUrl]
        },
        downloadRemoteIndexes: params[:downloadRemoteIndexes] == 'true',
        autoBlockActive: params[:autoBlockActive] == 'true',
        fileTypeValidation: params[:fileTypeValidation] == 'true',
        checksumPolicy: params[:checksumPolicy],
        artifactMaxAge: params[:artifactMaxAge].to_i,
        metadataMaxAge: params[:metadataMaxAge].to_i,
        itemMaxAge: params[:itemMaxAge].to_i
      }
    }
    # business logic in nexus
    # if params[:authentication] == 'on'
    payload = authentication_setting_in_proxy_repo_payload params, payload
    # end
    # if params[:connectionSettings] == 'on'
    payload = connection_setting_in_proxy_repo_payload params, payload
    # end
    payload
  end

  def self.authentication_setting_in_proxy_repo_payload(params,
                                                        payload)
    payload[:data][:remoteStorage][:authentication] = {}
    payload[:data][:remoteStorage][:authentication][:password] =
        params[:password]
    unless params[:username].empty?
      payload[:data][:remoteStorage][:authentication][:username] =
          params[:username]
    end
    unless params[:ntlmHost].empty?
      payload[:data][:remoteStorage][:authentication][:ntlmHost] =
          params[:ntlmHost]
    end
    unless params[:ntlmDomain].empty?
      payload[:data][:remoteStorage][:authentication][:ntlmDomain] =
          params[:ntlmDomain]
    end
    payload
  end

  def self.connection_setting_in_proxy_repo_payload(params,
                                                    payload)
    connectionsettings =
        payload[:data][:remoteStorage][:connectionSettings] = {}
    unless params[:connectionTimeout].empty?
      connectionsettings[:connectionTimeout] =
          params[:connectionTimeout].to_i
    end
    unless params[:retrievalRetryCount].empty?
      connectionsettings[:retrievalRetryCount] =
          params[:retrievalRetryCount].to_i
    end
    unless params[:queryString].empty?
      connectionsettings[:queryString] =
          params[:queryString]
    end
    unless params[:userAgentString].empty?
      connectionsettings[:userAgentString] =
          params[:userAgentString]
    end
    payload
  end

  def self.create_orchestrate_config_payload(params)
    payload = {
      data: {
        object_id: params[:object_id], # this always was true for virtual repo
        source_association_key: params[:source_association_key],
        queue_server: params[:queue_server],
        queue_username: params[:queue_username],
        queue_password: params[:queue_password],
        queue_name: params[:queue_name]
      }
    }
    payload
  end

  def self.delete_repository(params)
    begin
      endpoint_url = Services::SoapService
        .get_endpoint_url_for_iaf_app params[:sessionId],
                                      params[:iafappname]
      RestClient::Resource.new(
          format('%s/ctf/repositories-in-project/%s/%s',
                 endpoint_url,
                 params[:projectId],
                 params[:repoId]),
          headers: http_headers(params[:sessionId]),
          verify_ssl: OpenSSL::SSL::VERIFY_NONE
      ).put(' ')
      response = Status.new(true, 204)
    rescue RestClient::ResourceNotFound, RestClient::RequestFailed,
           RestClient::Unauthorized => e
      response = rescue_from_request_failed(e)
    rescue Errno::ECONNREFUSED, Errno::EHOSTUNREACH => e
      response = rescue_from_connection_error(e)
    rescue StandardError => e
      response = rescue_from_unknown_error(e)
    end
    response
  end

  def self.report_orchestrate_config(params)
    begin
      endpoint_url = Services::SoapService
        .get_endpoint_url_for_iaf_app params[:sessionId],
                                      params[:iafappname]
      RestClient::Resource.new(
          format('%s/ctf/orchestrate/%s/%s',
                 endpoint_url,
                 params[:projectId],
                 params[:repoId]),
          headers: http_headers(params[:sessionId]),
          verify_ssl: OpenSSL::SSL::VERIFY_NONE
      ).post create_orchestrate_config_payload(params).to_json
      response = Status.new(true, 204)
    rescue RestClient::ResourceNotFound, RestClient::RequestFailed,
           RestClient::Unauthorized => e
      response = rescue_from_request_failed(e)
    rescue Errno::ECONNREFUSED, Errno::EHOSTUNREACH => e
      response = rescue_from_connection_error(e)
    rescue StandardError => e
      response = rescue_from_unknown_error(e)
    end
    response
  end

  def self.delete_report_orchestrate_config(params)
    begin
      endpoint_url = Services::SoapService
        .get_endpoint_url_for_iaf_app params[:sessionId],
                                      params[:iafappname]
      RestClient::Resource.new(
          format('%s/ctf/orchestrate-config-delete/%s/%s',
                 endpoint_url,
                 params[:projectId],
                 params[:repoId]),
          headers: http_headers(params[:sessionId]),
          verify_ssl: OpenSSL::SSL::VERIFY_NONE
      ).put(' ')
      response = Status.new(true, 204)
    rescue RestClient::ResourceNotFound, RestClient::RequestFailed,
           RestClient::Unauthorized => e
      response = rescue_from_request_failed(e)
    rescue Errno::ECONNREFUSED, Errno::EHOSTUNREACH => e
      response = rescue_from_connection_error(e)
    rescue StandardError => e
      response = rescue_from_unknown_error(e)
    end
    response
  end

  def self.link_repository(params)
    begin
      endpoint_url = Services::SoapService
        .get_endpoint_url_for_iaf_app params[:sessionId],
                                      params[:iafappname]
      RestClient::Resource.new(
          format('%s/ctf/repositories-not-in-project/%s/%s',
                 endpoint_url,
                 params[:projectId],
                 params[:repoId]),
          headers: http_headers(params[:sessionId]),
          verify_ssl: OpenSSL::SSL::VERIFY_NONE
      ).post({})
      response = Status.new(true, 204,data:{})
    rescue RestClient::ResourceNotFound, RestClient::RequestFailed,
           RestClient::Unauthorized => e
      response = rescue_from_request_failed(e)
    rescue Errno::ECONNREFUSED, Errno::EHOSTUNREACH => e
      response = rescue_from_connection_error(e)
    rescue StandardError => e
      response = rescue_from_unknown_error(e)
    end
    response
  end

  def self.list_repositories(params)
    begin
      endpoint_url = Services::SoapService
        .get_endpoint_url_for_iaf_app params[:sessionId],
                                      params[:iafappname]
      repositories_string = RestClient::Resource.new(
          format('%s/ctf/repositories', endpoint_url),
          headers: http_headers(params[:sessionId]),
          verify_ssl: OpenSSL::SSL::VERIFY_NONE
      ).get
      repositories_json = JSON.parse(repositories_string)
      response = Status.new(true,
                            200,
                            message: 'Repositories List obtained',
                            data: repositories_json['data'])
    rescue RestClient::RequestFailed, RestClient::Unauthorized => e
      response = rescue_from_request_failed(e)
    rescue Errno::ECONNREFUSED, Errno::EHOSTUNREACH => e
      response = rescue_from_connection_error(e)
    rescue StandardError => e
      response = rescue_from_unknown_error(e)
    end
    response
  end

  def self.list_repositories_for_project(params)
    begin
      project_id = params[:projectId]
      endpoint_url = Services::SoapService
        .get_endpoint_url_for_iaf_app params[:sessionId],
                                      params[:iafappname]
      repositories_string = RestClient::Resource.new(
          format('%s/ctf/repositories-in-project/%s', endpoint_url, project_id),
          headers: http_headers(params[:sessionId]),
          verify_ssl: OpenSSL::SSL::VERIFY_NONE
      ).get
      repositories_json = JSON.parse(repositories_string)
      response = Status.new(true,
                            200,
                            message: 'Repositories List obtained',
                            data: repositories_json['data'])
    rescue RestClient::RequestFailed, RestClient::Unauthorized => e
      response = rescue_from_request_failed(e)
    rescue Errno::ECONNREFUSED, Errno::EHOSTUNREACH => e
      response = rescue_from_connection_error(e)
    rescue StandardError => e
      response = rescue_from_unknown_error(e)
    end
    response
  end

  def self.rescue_from_request_failed(e)
    if e.http_code == 403
      Status.new(false, 403, message: 'Permission denied')
    else
      begin
        if e.response
          error_json = JSON.parse(e.response)
          Status.new(false, e.http_code, message: error_json['errors'][0]['msg'])
        else
          Status.new(false, e.http_code, message: e.message)
        end
      rescue JSON::ParserError
        Status.new(false, e.http_code, message: e.response)
      end
    end
  end

  def self.rescue_from_connection_error(e)
    Status.new(false,
               503,
               message: 'Error! Probably the Binary server is down.
                         Please contact Teamforge administrator.
                         The error message is as follows ...
                         <br/>' +
                         e.to_s)
  end

  def self.rescue_from_unknown_error(e)
    Status.new(false,
               500,
               message: 'Error! An unexpected error occurred.
                         The error message is as follows ...
                         <br/>' +
                         e.to_s)
  end

  def self.http_headers(session_id)
    {
      content_type: :json,
      accept: :json,
      'X-Auth-Token' => session_id
    }
  end
end
