##
# class to abstract Orchestrate REST calls
#
class Services::OrchestrateService
  SCHEMA_ID = 'binary_artifact'
  SCHEMA_VERSION = 2

  def self.schema_exists?(ctf_soapsession_id)
    get orc_rest_endpoint(nil, :schema, SCHEMA_ID), ctf_soapsession_id
  rescue RestClient::ResourceNotFound
    return false
  end

  def self.create_schema(ctf_soapsession_id)
    resp = post orc_rest_endpoint(nil, :schema),
                schema_data, ctf_soapsession_id
    if resp.code.to_s =~ /^2/
      sleep 0.5
      30.times do
        resp = schema_exists?(ctf_soapsession_id)
        return resp if resp
        sleep 1
      end
      return false
    end
  end

  def self.tool_exists?(ctf_soapsession_id, ctf_project_url)
    iaf_app_name = Rails.configuration.teamforge.fetch :iaf_app_name
    tools = get orc_rest_endpoint(ctf_project_url, :tool),
                ctf_soapsession_id
    if tools['tools'] && tools['tools'].size > 0
      binary_tool = tools['tools'].find { |tool| tool['service_name'] == iaf_app_name }
      if binary_tool
        return { 'tool' =>  binary_tool}
      end
    end
  rescue RestClient::ResourceNotFound
    return tools
  end

  def self.create_tool(ctf_soapsession_id, ctf_project_url)
    iaf_app_name = Rails.configuration.teamforge.fetch :iaf_app_name
    resp = post orc_rest_endpoint(ctf_project_url, :tool),
                tool_data(iaf_app_name), ctf_soapsession_id
    return JSON.parse(resp) if resp.code.to_s =~ /^2/
  end

  def self.fetch_source(ctf_soapsession_id, ctf_project_url, source_id)
    get(orc_rest_endpoint(ctf_project_url, :source, source_id),
        ctf_soapsession_id)['source']
  rescue RestClient::ResourceNotFound
    return
  end

  def self.create_source(ctf_soapsession_id, ctf_project_url, source_name,
                         tool_id, schema_store_id, associated_source)
    resp = post orc_rest_endpoint(ctf_project_url, :source),
                source_data(source_name, tool_id, schema_store_id,
                            associated_source),
                ctf_soapsession_id
    return JSON.parse(resp) if resp.code.to_s =~ /^2/
  end

  def self.deactivate_source(ctf_soapsession_id, ctf_project_url, source_id)
    resp = put orc_rest_endpoint(ctf_project_url, :source, source_id, false),
               '{}', ctf_soapsession_id
    true if resp.code.to_s =~ /^2/
  end

  def self.reactivate_source(ctf_soapsession_id, ctf_project_url, source_id)
    resp = put orc_rest_endpoint(ctf_project_url, :source, source_id, true),
               '{}', ctf_soapsession_id
    true if resp.code.to_s =~ /^2/
  end

  def self.update_associated_source(ctf_soapsession_id, ctf_project_url,
                                    source_id, associated_source_id)
    associated_source_id = '' if associated_source_id.nil?
    resp = put orc_rest_endpoint(ctf_project_url, :source, source_id),
               '{"associated_activity_source_id":"' + associated_source_id +
               '"}',
               ctf_soapsession_id
    true if resp.code.to_s =~ /^2/
  end

  def self.fetch_build_sources(ctf_soapsession_id, ctf_project_url)
    fetch_sources(ctf_soapsession_id, ctf_project_url) do |s|
      s['_type'] == 'Build' && s['active']
    end
    .try { |sources| sources.sort_by { |s| s['display_name'] } }
  end

  def self.fetch_source_by_sak(ctf_soapsession_id, ctf_project_url, sak)
    fetch_sources(ctf_soapsession_id, ctf_project_url) do |s|
      s['source_association_key'] == sak
    end
    .try { |source| source[0] }
  end

  class << self
    private

    def get(endpoint, token)
      JSON.parse rest_resource(endpoint, token).get
    end

    def post(endpoint, data, token)
      rest_resource(endpoint, token).post data
    end

    def put(endpoint, data, token)
      rest_resource(endpoint, token).put data
    end

    def rest_resource(endpoint, token)
      RestClient::Resource.new(
        endpoint,
        headers: http_headers(token),
        verify_ssl: OpenSSL::SSL::VERIFY_NONE)
    end

    def http_headers(session_id)
      {
        content_type: :json,
        accept: :json,
        'X-Orchestrate-Session' => session_id
      }
    end

    def orc_rest_endpoint(ctf_project_url, what, key = nil, activate = nil)
      orc_api_url = Rails.configuration.teamforge.fetch :orc_api_url
      case what.to_sym
      when :schema
        "#{orc_api_url}/schemas" +
          (key.nil? ? '' : "/#{key}")
      when :source
        "#{orc_api_url}/projects/#{ctf_project_url}/sources" +
          (key.nil? ? '' : "/#{key}") +
          case activate
          when nil then ''
          when true then '/activate'
          when false then '/deactivate'
          end
      when :tool
        "#{orc_api_url}/projects/#{ctf_project_url}/tools" +
          (key.nil? ? '' : "?service_name=#{key}")
      end
    end

    def schema_data
      <<-EOF
      {
        "api_version": "1",
        "custom_schema": {
          "event_time": "#{Time.now.getgm.iso8601}",
          "name" : "Binaries",
          "schema_id" : "#{SCHEMA_ID}",
          "schema_version" : #{SCHEMA_VERSION},
          "fields" : [
              "event_type",
              "operation",
              "event_time",
              "schema_id",
              "schema_version",
              "remote_id",
              "name",
              "description",
              "build_number",
              "associated_remote_id",
              "repository_id",
              "repository_url",
              "artifact_url",
              "group_id",
              "artifact_id",
              "artifact_version",
              "artifact_type",
              "artifact_classifier",
              "link_to",
              "component",
              "component_of",
              "created_by"
          ],
          "important" : [],
          "parsed" : [],
          "required" : [
              "remote_id",
              "event_type",
              "operation",
              "event_time",
              "schema_id",
              "schema_version",
              "link_to"
          ],
          "hidden" : [
              "repository_url",
              "artifact_url"
          ]
        }
      }
      EOF
    end

    def tool_data(iaf_app_name)
      <<-EOF
      {
        "service_name": "#{iaf_app_name}",
        "name": "#{iaf_app_name}"
      }
      EOF
    end

    def source_data(source_name, tool_id, schema_store_id,
                    associated_source_id = nil)
      <<-EOF.squish
      {
        "_type": "XDS",
        "display_name": "#{source_name}",
        "tool_id": "#{tool_id}",
        "icon_uri": "/assets/icons/custom_source/XDS_Icon_04.png",
        "background_color_value" : "#ffffff",
        #{if associated_source_id =~ /\S/
            '"associated_activity_source_id": "' + associated_source_id + '",'
          else
            ''
          end
        }
        "custom_schema_store_id": "#{schema_store_id}"
      }
      EOF
    end

    def fetch_sources(ctf_soapsession_id, ctf_project_url)
      resp = get orc_rest_endpoint(ctf_project_url, :source),
                 ctf_soapsession_id
      if block_given?
        resp['sources'].select { |s| yield s }
      else
        resp['sources']
      end
    rescue
      return
    end
  end
end
