#
# Copyright 2014 CollabNet, Inc. All rights reserved. Usage governed by the
# LICENSE.txt file.
#

# Connects app to Teamforge instance via IAF

class CtfConfigurer
  require 'teamforge'
  INTEGRATION_DATA_NAMESPACE = 'IAFLandingApp.Enabled'
  VERSION = '15.12'

  MIGRATIONS = {
    '15.12' => [:send_schema]
  }

  def initialize(ctf_admin_user, ctf_admin_pass)
    @options = Rails.configuration.teamforge
    Rails.logger.debug "teamforge config is #{@options.inspect}"

    # Use fetch so we raise an exception if key not found
    @app_name = @options.fetch(:iaf_app_name)
    @app_name_lc = @app_name.downcase
    TeamForge.use_ssl = @options[:ctf_base_url].starts_with?('https')
    TeamForge.server = @options[:ctf_base_url]
    Rails.logger.debug "Server is #{TeamForge.server}"

    @session = TeamForge::Login.new(ctf_admin_user, ctf_admin_pass).request
    @pluggable = TeamForge::PluggableApp
    @simple_file_storage = TeamForge::SimpleFileStorageApp
    @integration_data = TeamForge::IntegrationDataApp

    # This is a bit messy but works and it works thus:
    # Where your setup is behind a reverse proxying apache shared with CTF,
    # iaf_host_url is same as ctf_base_url, relative_url_root is the
    # subdirectory proxied (e.g. /iaf) and url_prefix is binary. When not behind
    # such a setup, iaf_host_url is just the place where rails runs,
    # and relative_url_root is not set so it works then too.
    @url_prefix = Rails.configuration.teamforge[:iaf_host_url].chomp('/') +
                  Rails.configuration.relative_url_root.chomp('/') +
                  '/' + Rails.configuration.teamforge[:app_url_prefix]
  end

  # Configures a TeamForge instance with this app, optionally adding the app to
  # all existing projects if instructed
  def configure(add_to_site = true, update_app = false, add_to_projects = false)
    begin
      iaf_app_info = iaf_application
      iaf_app_info = add_app_to_site(iaf_app_info) if add_to_site
      update_app_in_site(iaf_app_info) if update_app
    rescue Exception => e
      if e.message.include?('Binaries seems to be down.')
        Rails.logger.info "Modifying binary integrated app"
        plug_id = ENV['BINARY_APP_ID']
        icon_key = ENV['BINARY_ICON_KEY']
        edit_iaf_application(plug_id, icon_key) if update_app && plug_id && icon_key
      end
    end
      
    return unless add_to_projects

    projects_list = TeamForge::GetProjectList.new(@session, false).request
    projects_list.each do |project|
      Rails.logger.info "Enabling app for project #{project.title} if required"
      add_to_project(iaf_app_info.id, project.id, iaf_app_info.prefix)
    end
  end

  def add_app_to_site(iaf_app_info)
    if iaf_app_info
      Rails.logger.info "App #{@app_name} already exists for site"
      Rails.logger.info 'Use edit option if you want to refresh it'
    else
      Rails.logger.info "Registering app #{@app_name} for site"
      iaf_app_info = create_iaf_application
    end
    iaf_app_info
  end

  def update_app_in_site(iaf_app_info)
    if iaf_app_info
      Rails.logger.info "Refreshing app #{@app_name} for site"
      iaf_app_info = edit_iaf_application(iaf_app_info.id, iaf_app_info.icon_key)
    else
      Rails.logger.info "App #{@app_name} does not exist for site"
      Rails.logger.info 'Use configure option if you want to add it'
    end
    iaf_app_info
  end

  def send_schema
      Services::OrchestrateService.create_schema(@session)
    rescue Exception => ex
      Rails.logger.warn "Create Schema failed #{ex}"
  end

  # Uploads the IAF App icon into TeamForge for use in the project toolbar
  def upload_icon
    file_storage_id =
      @simple_file_storage::StartFileUpload.new(@session).request
    icon_path = Rails.root + "app/assets/images/#{@app_name_lc}-icon.png"
    icon_encoded = Base64.encode64(File.binread(icon_path))
    @simple_file_storage::Write
      .new(@session, file_storage_id, icon_encoded)
      .request
    @simple_file_storage::EndFileUpload.new(@session, file_storage_id).request

    file_storage_id
  end

  # Creates the application inside TeamForge. This is the first step in adding
  # app to a teamforge project.
  def create_iaf_application
    icon_file_key = upload_icon
    dep_xml, app_xml = read_xml_files
    @pluggable::CreateIntegratedApplicationFromXMLFiles5
      .new(@session, dep_xml, app_xml, icon_file_key,
           "#{@app_name_lc}-icon.png", 'image/png')
      .request
  end

  # sets the integrated application icon for a given integrated app
  # it is kept separate instead of coupling with edit as in UI they dont
  # have a provision to upload icon. So till that feature shows up let this
  # be separate so that users can call it when ever needed.
  def set_integrated_app_icon
    Rails.logger.info 'Setting Integrated Application icon process in progress'
    icon_file_key = upload_icon
    integrated_app_id = @pluggable::GetIntegratedApplicationByName4
      .new(@session, @app_name).request.id
    @pluggable::SetIntegratedApplicationIcon
      .new(@session,
           integrated_app_id,
           icon_file_key,
           "#{@app_name_lc}-icon.png",
           'image/png').request
    Rails.logger.info(
      'Setting Integrated Application icon process was successful')
  end

  # Refreshes the application inside TeamForge. Used xml changes. Unfortunately
  # cannot be used for icons as the API is kind of broken (the icon_file_key
  # expected here is not what simple_file_storage returns!
  def edit_iaf_application(plug_id, icon_key)
    dep_xml, app_xml = read_xml_files
    @pluggable::EditIntegratedApplicationFromXMLFiles
      .new(@session, plug_id, dep_xml, app_xml, icon_key)
      .request
  end

  def read_xml_files
    dep_xml_file = Rails.root + "xml-descriptors/#{@app_name_lc}-deploy.xml"
    app_xml_file = Rails.root +
                   "xml-descriptors/#{@app_name_lc}-application.xml"
    dep_xml = Base64.encode64(ERB.new(File.read dep_xml_file).result binding)
    app_xml = Base64.encode64(ERB.new(File.read app_xml_file).result binding)

    [dep_xml, app_xml]
  end

  # Gets IAF app details
  def iaf_application
    @pluggable::GetIntegratedApplicationByName4
      .new(@session, @app_name)
      .request
  rescue TeamForge::TeamForgeError => e
    Rails.logger.debug(e.message)
    if e.message =~ /NoSuchObjectFault/
      return nil
    else
      raise e
    end
  end

  # Adds the app to a project
  def add_to_project(pluggable_id, project_id, prefix)
    # :session_id, :project_id, :pluggable_id, :params, :prefix)
    @pluggable::EnablePluggableComponent.new(@session, project_id, pluggable_id,
                                             [], prefix).request
  end

  # Adds to integration data. This sets up data that will be used by project
  # creation event handler to decide which IAF landing apps must be auto enabled
  def setup_integration_data(enabled) # enabled == Y|N
    # First, check if namespace exists or register one
    namespace_id =
      begin
        @integration_data::GetNamespaceId
            .new(@session, INTEGRATION_DATA_NAMESPACE).request
      rescue TeamForge::TeamForgeError
        @integration_data::RegisterNamespace
            .new(@session, INTEGRATION_DATA_NAMESPACE).request
      end
    Rails.logger.info "Setting up integration data #{@app_name}='#{enabled}'"
    @integration_data::PutDataByKey
          .new(@session, namespace_id, 'proj1001', @app_name,  enabled).request
  end

  # Call migrations upto current version
  def migrate
    conf = ConfigurationState.first
    last_version = conf.version
    return unless VERSION > last_version

    MIGRATIONS.keys.sort.each do |v|
      next if v <= last_version

      MIGRATIONS[v].each do |meth|
        Rails.logger.info "Migration(#{v}): Running #{meth}"
        send meth
      end
    end
    conf.version = VERSION
    Rails.logger.debug "Setting configuration_state version to ${VERSION}"
    conf.save!
  end
end
